using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Threading;
using Microsoft.Surface;
using Microsoft.Surface.Presentation;
using Microsoft.Surface.Presentation.Controls;

namespace FingerPaint
{
    /// <summary>
    /// Interaction logic for SurfaceWindow1.xaml
    /// </summary>
    public partial class SurfaceWindow1 : SurfaceWindow
    {
        RenderTargetBitmap bitmap;
        DateTime dtStart;

        /// <summary>
        /// Default constructor.
        /// </summary>
        public SurfaceWindow1()
        {
            InitializeComponent();

            // Add handlers for Application activation events
            AddActivationHandlers();

            bitmap = new RenderTargetBitmap(768, 768, 96, 96, PixelFormats.Default);
            image.Source = bitmap;

            Contacts.EnableInput(this);
            dtStart = DateTime.Now;

            CompositionTarget.Rendering += OnCompositionTargetRendering;


            

            
        }

        

        class Info
        {
            public bool StillDown;
            public Point Center;
            public double MajorAxis;
            public double MinorAxis;
            public double Orientation;
        }

        Brush brush = new SolidColorBrush(Colors.Blue);
        Dictionary<int, Info> infoDict = new Dictionary<int, Info>();

        void OnCompositionTargetRendering(object sender, EventArgs args)
        {
            double seconds = (DateTime.Now - dtStart).TotalSeconds;
            rotate.Angle = 360 * seconds / 5;

            // Calculate color and brush
            Color clr;
            double fraction = 6 * (seconds % 10) / 10;

            if (fraction < 1)
                clr = Color.FromRgb(0xFF, (byte)(fraction * 255), 0);
            else if (fraction < 2)
                clr = Color.FromRgb((byte)(255 - (fraction - 1) * 255), 255, 0);
            else if (fraction < 3)
                clr = Color.FromRgb(0, 255, (byte)((fraction - 2) * 255));
            else if (fraction < 4)
                clr = Color.FromRgb(0, (byte)(255 - (fraction - 3) * 255), 255);
            else if (fraction < 5)
                clr = Color.FromRgb((byte)((fraction - 4) * 255), 0, 255);
            else
                clr = Color.FromRgb(255, 0, (byte)(255 - (fraction - 5) * 255));

            ReadOnlyContactCollection contacts = Contacts.GetContactsOver(this);

            if (contacts.Count > 0)
                brush = new SolidColorBrush(clr);

            foreach (Contact contact in contacts)
            {
                Info info2 = new Info();
                info2.Center = contact.GetCenterPosition(testGrid);

                double majorAxis, minorAxis, orientation;
                contact.GetEllipse(testGrid, out majorAxis, out minorAxis, out orientation);

                info2.MajorAxis = (majorAxis + minorAxis) / 2;
                info2.MinorAxis = (majorAxis + minorAxis) / 2;
                info2.Orientation = orientation;
                        
                Info info1 = contact.GetUserData(0) as Info;

                if (info1 != null)
                {
                    Render(info1, info2);
                }

                contact.SetUserData(0, info2);
            }
        }

        ScaleTransform none = new ScaleTransform();
        ScaleTransform flipX = new ScaleTransform(-1, 1, 384, 384);
        ScaleTransform flipY = new ScaleTransform(1, -1, 384, 384);
        ScaleTransform flipXY = new ScaleTransform(-1, -1, 384, 384);

        void Render(Info info1, Info info2)
        {
            DrawingVisual visual = new DrawingVisual();
            DrawingContext dc = visual.RenderOpen();

            Render(dc, none, info1, info2);
            Render(dc, flipX, info1, info2);
            Render(dc, flipY, info1, info2);
            Render(dc, flipXY, info1, info2);

            dc.Close();
            bitmap.Render(visual);
        }

        void Render(DrawingContext dc, Transform xform, Info info1, Info info2)
        {
            dc.PushTransform(xform);

            DrawEllipse(dc, info1);
            DrawEllipse(dc, info2);
            DrawLine(dc, info1, info2);

            dc.Pop();
        }

        void DrawEllipse(DrawingContext dc, Info info)
        {
            dc.DrawEllipse(brush, null, info.Center, info.MajorAxis, info.MinorAxis);
        }

        void DrawLine(DrawingContext dc, Info info1, Info info2)
        {
            double radius1 = (info1.MajorAxis + info1.MinorAxis) / 2;
            double radius2 = (info2.MajorAxis + info2.MinorAxis) / 2;

            Vector lineVector = info2.Center - info1.Center;
            lineVector.Normalize();

            Point pt1 = info1.Center + radius1 * new Vector(lineVector.Y, -lineVector.X);
            Point pt2 = info2.Center + radius2 * new Vector(lineVector.Y, -lineVector.X);
            Point pt3 = info2.Center + radius2 * new Vector(-lineVector.Y, lineVector.X);
            Point pt4 = info1.Center + radius1 * new Vector(-lineVector.Y, lineVector.X);

            PathGeometry geometry = new PathGeometry();
            PathFigure figure = new PathFigure();
            figure.StartPoint = pt1;
            geometry.Figures.Add(figure);
            PolyLineSegment segment = new PolyLineSegment();
            segment.Points.Add(pt2);
            segment.Points.Add(pt3);
            segment.Points.Add(pt4);
            figure.Segments.Add(segment);

            dc.DrawGeometry(brush, null, geometry);
        }

        /// <summary>
        /// Occurs when the window is about to close. 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnClosed(EventArgs e)
        {
            base.OnClosed(e);

            // Remove handlers for Application activation events
            RemoveActivationHandlers();
        }

        /// <summary>
        /// Adds handlers for Application activation events.
        /// </summary>
        private void AddActivationHandlers()
        {
            // Subscribe to surface application activation events
            ApplicationLauncher.ApplicationActivated += OnApplicationActivated;
            ApplicationLauncher.ApplicationPreviewed += OnApplicationPreviewed;
            ApplicationLauncher.ApplicationDeactivated += OnApplicationDeactivated;
        }

        /// <summary>
        /// Removes handlers for Application activation events.
        /// </summary>
        private void RemoveActivationHandlers()
        {
            // Unsubscribe from surface application activation events
            ApplicationLauncher.ApplicationActivated -= OnApplicationActivated;
            ApplicationLauncher.ApplicationPreviewed -= OnApplicationPreviewed;
            ApplicationLauncher.ApplicationDeactivated -= OnApplicationDeactivated;
        }

        /// <summary>
        /// This is called when application has been activated.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnApplicationActivated(object sender, EventArgs e)
        {
            //TODO: enable audio, animations here
        }

        /// <summary>
        /// This is called when application is in preview mode.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnApplicationPreviewed(object sender, EventArgs e)
        {
            //TODO: Disable audio here if it is enabled

            //TODO: optionally enable animations here
        }

        /// <summary>
        ///  This is called when application has been deactivated.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnApplicationDeactivated(object sender, EventArgs e)
        {
            //TODO: disable audio, animations here
        }

    }
}